import QUB
from Buttons import *
from qubtree import Node
import math
import win32com.client
from pythoncom import CoInitialize, CoUninitialize

### -------------------------------------------------------------------------
def Covariance( aData, nAvg, iSep) :
    n = 0.0
    for i in range( len(aData)-iSep ) :
        n = n + aData[i]*aData[i+iSep]
    return n/(len(aData)-iSep)

### -------------------------------------------------------------------------
### Calculate AutoCorrelation for selected idealized data 
def AutoCorrelation(props) : 
    #----- Properties
    iMaxDelta = props['K']
    cExcel = props['excel']

    #----- Scan data segments; Calc Sum and a average dwell times 
    aOpen = []
    aShut = []
    nSumOpen = 0.0
    nSumShut = 0.0
    
    xidl=QUB.getChosenIdealization()
    xseg=xidl.find('DataSet').find('Segment')
    while not xseg.isNull:
        xcla=xseg.find('Classes')
        xdur=xseg.find('Durations')
        for iSeg in range(xseg.find('DwellCount').data[0]) :
            # todo - extend to multiple classes ( xcla = 0, 1, 2, ... )
            nDur = xdur.data[iSeg]
            if xcla.data[iSeg]==0 :
                aShut.append( nDur ) #/1000.0 ?
                nSumShut = nSumShut + nDur
            else :  
                aOpen.append(xdur.data[iSeg]) #/1000.0 ?
                nSumOpen = nSumOpen + nDur 
        xseg = xseg.next('Segment')
        
    if len(aShut) > 0 : 
        nAvgShut = nSumShut / len(aShut)
    else :
        nAvgShut = 1
        
    if len(aOpen) > 0 : 
        nAvgOpen = nSumOpen / len(aOpen)
    else :
        nAvgOpen = 1

    #----- Calculate Variances
    aVarOpen = []
    for iSeg in range( len(aOpen) ) :
        aVarOpen.append( aOpen[iSeg]-nAvgOpen )
    aVarShut = []
    for iSeg in range( len(aShut) ) :
        aVarShut.append( aShut[iSeg]-nAvgShut )

    #----- Calculate covariances
    QUB.Report( '%d Shut,Ave=%f,sum=%f   ;   %d Open,Ave=%f,Sum=%f' % ( 
                    len(aShut), nAvgShut, nSumShut, 
                    len(aOpen), nAvgOpen, nSumOpen) )
    aCoVarOpen = []
    aCoVarShut = []
    for iSep in range(iMaxDelta) :
        aCoVarOpen.append( Covariance( aVarOpen, nAvgOpen, iSep ) )
        aCoVarShut.append( Covariance( aVarShut, nAvgShut, iSep ) )
        
        QUB.Report( 'k=%d   ShutCovariance=%f   OpenCovariance=%f ' % ( iSep, 
                        aCoVarShut[iSep]/aCoVarShut[0],
                        aCoVarOpen[iSep]/aCoVarOpen[0]) )

    #-----Output data to excel
    if cExcel=='Y' :
        nNoise = 2 / math.sqrt( len(aOpen)+len(aShut) )
        excel = win32com.client.Dispatch("Excel.Application")
        excel.Visible = 1
        book = excel.Workbooks.Add(-4167) # XLWBatWorksheet
        sheet = book.Worksheets.Item(1)
        
        cells = sheet.Cells
        iMax=0
        cells(1,2).Value = 'Shut'
        cells(1,3).Value = 'Open'
        cells(1,4).Value = 'Noise(+)'
        cells(1,5).Value = 'Noise(-)'
        for iSep in range(1, iMaxDelta) :
            cells(iSep+1,2).Value = aCoVarShut[iSep]/aCoVarShut[0]
            cells(iSep+1,3).Value = aCoVarOpen[iSep]/aCoVarOpen[0]
            cells(iSep+1,4).Value = nNoise
            cells(iSep+1,5).Value = -1.0 * nNoise 
            
        sheet.ChartObjects().Add(300,25,400,300)
        chart = sheet.ChartObjects(1).Chart
        chart.ChartType = 65 # xlLineMarkers  (xlLine=4) (xlLineStacked=63)
        chart.SeriesCollection().Add( sheet.Range( sheet.Cells(1,2), sheet.Cells(iMaxDelta,5) ) )
        chart.SeriesCollection(1).Border.Color = 0x999999
        chart.SeriesCollection(1).Border.Weight = 0x2 # xlThin

        excel.Visible = 1
            

### -------------------------------------------------------------------------
AddTaskButton( 'Correlation', # button text
                AutoCorrelation,  # function
                'Cor',       # short text for small buttons
                0,            # show settings dialog on left-click?
                1,            # show settings dialog on right-click?
                "AutoCorrelation",     # tool-tip
                "AutoCorrelation",     # description in "Edit Buttons"
                "Idealized Data AutoCorrelation :",    # caption for the settings dialog
                [TextField("K",  "K (separation) :", "5", acceptIntGreaterThan(-1)),
                 TextField("excel",  "Excel Output (Y/N) :", "Y")
                ],
                'Preprocessing' )

### -------------------------------------------------------------------------
### Calculate AutoCorrelation for selected idealized data 
def AutoSegCorrelation(props) : 
    #----- Properties
    iMaxDelta = props['K']
    iNLargest = props['nlargest']

    #----- Open excel
    excel = win32com.client.Dispatch("Excel.Application")
    excel.Visible = 1
    book = excel.Workbooks.Add(-4167) # XLWBatWorksheet
    sheet = book.Worksheets.Item(1)
    cells = sheet.Cells
    iXLRow = 1

    #----- Scan data segments; Calc Sum and a average dwell times 
    xidl=QUB.getChosenIdealization()

    #----- Find which segs to use
    aMax = []
    iSegment = 0 
                      
    xseg=xidl.find('DataSet').find('Segment')
    while not xseg.isNull:
        iSegment = iSegment + 1
        iLen = xseg.find('DwellCount').data[0]
        if len(aMax)<iNLargest :
            aMax.append( [iLen, iSegment] )
            aMax.sort()
        elif iLen>aMax[0][0] :
            aMax[0]=[iLen,iSegment]
            aMax.sort()
        QUB.Report(' seg=%d Dwells=%d Start=%d End=%d ' % ( iSegment, iLen, xseg.data[0], xseg.data[1] ) );
        xseg = xseg.next('Segment')

    #-----
    for i in range( len(aMax) ) :
        aMax[i]=aMax[i][1]
    aMax.sort()
        
    #----- 
    xseg=xidl.find('DataSet').find('Segment')
    iSegment=0
    iChart=0
    while not xseg.isNull and len(aMax)>0 :
        iSegment=iSegment+1
        if aMax[0]==iSegment : 
            aMax.remove(aMax[0])

            aOpen = []
            aShut = []
            nSumOpen = 0.0
            nSumShut = 0.0
                
            xcla=xseg.find('Classes')
            xdur=xseg.find('Durations')
            for iSeg in range(xseg.find('DwellCount').data[0]) :
                nDur = xdur.data[iSeg]
                if xcla.data[iSeg]==0 :
                    aShut.append( nDur ) #/1000.0 ?
                    nSumShut = nSumShut + nDur
                else :  
                    aOpen.append(xdur.data[iSeg]) #/1000.0 ?
                    nSumOpen = nSumOpen + nDur 

            if len(aShut) > 0 : 
                nAvgShut = nSumShut / len(aShut)
            else :
                nAvgShut = 1
                
            if len(aOpen) > 0 : 
                nAvgOpen = nSumOpen / len(aOpen)
            else :
                nAvgOpen = 1

            #----- Calculate Variances
            aVarOpen = []
            for iSeg in range( len(aOpen) ) :
                aVarOpen.append( aOpen[iSeg]-nAvgOpen )
            aVarShut = []
            for iSeg in range( len(aShut) ) :
                aVarShut.append( aShut[iSeg]-nAvgShut )

            #----- Calculate covariances
            QUB.Report( '%d Shut,Ave=%f,sum=%f   ;   %d Open,Ave=%f,Sum=%f' % ( 
                            len(aShut), nAvgShut, nSumShut, 
                            len(aOpen), nAvgOpen, nSumOpen) )
            aCoVarOpen = []
            aCoVarShut = []
            for iSep in range(iMaxDelta) :
                aCoVarOpen.append( Covariance( aVarOpen, nAvgOpen, iSep ) )
                aCoVarShut.append( Covariance( aVarShut, nAvgShut, iSep ) )
                #QUB.Report( 'k=%d   ShutCovariance=%f   OpenCovariance=%f ' % ( iSep, 
                #                aCoVarShut[iSep]/aCoVarShut[0],
                #                aCoVarOpen[iSep]/aCoVarOpen[0]) )
                
            #-----Output data to excel
            nNoise = 2 / math.sqrt( len(aOpen)+len(aShut) )
            iMax=0
            cells(iXLRow,1).Value = iSegment
            cells(iXLRow+1,1).Value = xseg.data[0] 
            cells(iXLRow+2,1).Value = xseg.data[1] 
            cells(iXLRow,2).Value = 'Shut'
            cells(iXLRow,3).Value = 'Open'
            cells(iXLRow,4).Value = 'Noise(+)'
            cells(iXLRow,5).Value = 'Noise(-)'
            for iSep in range(1, iMaxDelta) :
                cells(iSep+iXLRow,2).Value = aCoVarShut[iSep]/aCoVarShut[0]
                cells(iSep+iXLRow,3).Value = aCoVarOpen[iSep]/aCoVarOpen[0]
                cells(iSep+iXLRow,4).Value = nNoise
                cells(iSep+iXLRow,5).Value = -1.0 * nNoise 

            iRows = iMaxDelta
            if iRows < 10 :
                iRows = 10 
            
            sheet.ChartObjects().Add(300,iXLRow*12.75-6.375,400,(iRows)*12.75)
            iChart = iChart + 1 
            chart = sheet.ChartObjects(iChart).Chart
            chart.ChartType = 65 # xlLineMarkers  (xlLine=4) (xlLineStacked=63)
            chart.SeriesCollection().Add( sheet.Range( sheet.Cells(iXLRow,2), sheet.Cells(iXLRow+iMaxDelta-1,5) ) )
            chart.SeriesCollection(1).Border.Color = 0x999999
            chart.SeriesCollection(1).Border.Weight = 0x2 # xlThin

            iXLRow = iXLRow + iRows + 4

        xseg = xseg.next('Segment')
        
    excel.Visible = 1
           

### -------------------------------------------------------------------------
AddTaskButton( 'Cor2', # button text
                AutoSegCorrelation,  # function
                'Cor2',       # short text for small buttons
                0,            # show settings dialog on left-click?
                1,            # show settings dialog on right-click?
                "AutoSegCorrelation",     # tool-tip
                "AutoSegCorrelation",     # description in "Edit Buttons"
                "Idealized Data AutoCorrelation :",    # caption for the settings dialog
                [TextField("K",  "K (max separation) :", "5", acceptIntGreaterThan(-1)),
                 TextField("nlargest",  "Analyze the <n> longest segs :", "10", acceptIntGreaterThan(-1))
                ],
                'Preprocessing' )

### -------------------------------------------------------------------------
### Calculate Class Transitions
def Transition(props) : 
    cExcel = props['excel']

    #----- Transitions array 
    iClasses=1
    aTrans = [ [0] ]

    #----- 
    xidl=QUB.getChosenIdealization()
    xseg=xidl.find('DataSet').find('Segment')
    while not xseg.isNull :
        iPrevious=-1            
        xcla=xseg.find('Classes')
        #xdur=xseg.find('Durations')
        for iSeg in range(xseg.find('DwellCount').data[0]) :
            iClass = xcla.data[iSeg]
            if iClass >= iClasses :
                while iClass >= iClasses :
                    aTrans.append( [0,0] )
                    iClasses=iClasses+1
                for i in range(iClasses) :
                    while len(aTrans[i])<iClasses :
                        aTrans[i].append( 0 )
            if iPrevious >= 0 :
                aTrans[iPrevious][iClass]=aTrans[iPrevious][iClass]+1
            iPrevious=iClass
        xseg = xseg.next('Segment')

    if cExcel=='Y' : 
        #----- Open excel
        excel = win32com.client.Dispatch("Excel.Application")
        excel.Visible = 1
        book = excel.Workbooks.Add(-4167) # XLWBatWorksheet
        sheet = book.Worksheets.Item(1)
        cells = sheet.Cells

        #-----Output data to excel
        cells(3,1).Value='From'
        cells(1,3).Value='To'
        for i in range(iClasses) :
             cells(i+3,2).Value=i
             cells(2,i+3).Value=i
             for j in range(iClasses) :
                 cells(i+3,j+3).Value=aTrans[i][j]
        excel.Visible = 1
    else :
        for i in range(iClasses) :
            QUB.Report( aTrans[i] )

### -------------------------------------------------------------------------
AddTaskButton( 'Transit', # button text
                Transition,  # function
                'Trn',       # short text for small buttons
                0,            # show settings dialog on left-click?
                1,            # show settings dialog on right-click?
                "Transition between classes",     # tool-tip
                "Transition between classes",     # description in "Edit Buttons"
                "Idealized Data Transitions :",    # caption for the settings dialog
                [TextField("excel",  "Excel Output (Y/N) :", "Y")
                ],
                'Preprocessing' )

